@@ -3,14 +3,14 @@
 from typing import List
 from typing import Dict
 from typing import Union
-
-from pydantic import Field
-from inspect import signature
-from makefun import create_function
+from typing import Optional
+from inflection import singularize
+from inflection import titleize
 
 from fastapi import APIRouter
 from fastapi import Request
 from fastapi import Depends
+from fastapi import BackgroundTasks
 from fastapi.responses import JSONResponse
 
 from tendril.authn.users import auth_spec
@@ -19,21 +19,24 @@ from tendril.authn.users import authn_dependency
 
 from tendril.authn.pydantic import UserReferenceTModel
 from tendril.authz.roles.interests import MembershipInfoTModel
-from tendril.common.interests.states import InterestLifecycleStatus
+from tendril.common.states import LifecycleStatus
 from tendril.utils.db import get_session
+from tendril.common.interests.representations import ExportLevel
 
 from .base import ApiRouterGenerator
+from tendril.utils import log
+logger = log.get_logger(__name__, log.DEFAULT)
 
 
 class InterestLibraryRouterGenerator(ApiRouterGenerator):
     def __init__(self, actual):
         super(InterestLibraryRouterGenerator, self).__init__()
         self._actual = actual
+        self._item_tmodel = None
 
     async def items(self, request: Request,
                     user: AuthUserModel = auth_spec(),
-                    include_roles: bool = False,
-                    include_permissions: bool = False,
+                    export_level: Optional[ExportLevel] = ExportLevel.STUB,
                     include_inherited: bool = True):
         """
         Get a list of all items in this library.
@@ -49,22 +52,17 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
 
          - **user :** The requesting user, identified by the access token, whose list of
                       interests is to be provided.
-         - **include_roles :** Include the user's roles in the response.
-         - **include_permissions :** Include the user's permissions in the response.
          - **include_inherited : ** Include interests in which the user's access inherited.
         """
         with get_session() as session:
             rv = [x.export(auth_user=user, session=session,
-                           include_roles=include_roles,
-                           include_permissions=include_permissions)
+                           export_level=export_level)
                   for x in self._actual.items(user=user, session=session,
                                               include_inherited=include_inherited,)]
         return rv
 
     async def new_items(self, request: Request,
-                        user: AuthUserModel = auth_spec(),
-                        include_roles: bool = False,
-                        include_permissions: bool = False):
+                        user: AuthUserModel = auth_spec()):
         """
         Get a list of all new items in this library.
 
@@ -81,17 +79,14 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
          - **include_permissions :** Include the user's permissions in the response.
         """
         with get_session() as session:
-            rv = [x.export(session=session,
-                           include_roles=include_roles,
-                           include_permissions=include_permissions)
-                  for x in self._actual.items(state=InterestLifecycleStatus.NEW,
+            rv = [x.export(session=session, export_level=ExportLevel.STUB)
+                  for x in self._actual.items(state=LifecycleStatus.NEW,
                                               session=session)]
         return rv
 
     async def item(self, request: Request, id: int,
                    user: AuthUserModel = auth_spec(),
-                   include_roles: bool = False,
-                   include_permissions: bool = False):
+                   export_level: Optional[ExportLevel] = ExportLevel.NORMAL):
         """
         Get a specific item from this library.
 
@@ -106,28 +101,34 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
 
           - **id :** The id of the interest to retrieve
           - **user :** The requesting user, identified by the access token.
-          - **include_roles :** Include the user's roles in the response.
-          - **include_permissions :** Include the user's permissions in the response.
         """
         with get_session() as session:
             rv = self._actual.item(id=id, session=session).\
-                export(auth_user=user, session=session,
-                       include_roles=include_roles,
-                       include_permissions=include_permissions)
+                export(auth_user=user, session=session, export_level=export_level)
         return rv
 
-    def _inject_create_model(self, ep, param='item'):
-        orig_sig = signature(ep)
-        params = list(orig_sig.parameters.values())
-        orig_param = next(p for p in params if p.name == param)
-        index = params.index(orig_param)
-        new_param = orig_param.replace(name=orig_param.name, default=orig_param.default, kind=orig_param.kind,
-                                       annotation=self._actual.interest_class.tmodel_create)
-        params[index] = new_param
-        new_sig = orig_sig.replace(parameters=params)
-        return create_function(func_signature=new_sig, func_impl=ep)
+    async def find_possible_parents(self, request: Request,
+                                    user: AuthUserModel = auth_spec(),
+                                    export_level: Optional[ExportLevel] = ExportLevel.STUB.value):
+        """
+        Get possible parents for new items in this library.
 
-    def create_item(self, item, user: AuthUserModel = auth_spec()):
+        All user memberships in possible parents for this library's interest
+        type which allow creation of a child of this type are returned.
+
+        - **user :* The requesting user, identified by the access token.
+        """
+        with get_session() as session:
+            result = self._actual.possible_parents(user=user, session=session)
+            return [x.export(auth_user=user, session=session,
+                             export_level=export_level) for x in result]
+
+    def _inject_create_model(self, ep):
+        return self._inject_model(ep, param='item', model=self._actual.interest_class.tmodel_create)
+
+    def create_item(self, item,
+                    user: AuthUserModel = auth_spec(),
+                    export_level: Optional[ExportLevel] = ExportLevel.STUB.value):
         """
         Create an Interest.
 
@@ -150,10 +151,23 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
         """
         with get_session() as session:
             item = self._actual.add_item(item, session=session)
-            rv = item.export(session=session)
+            rv = item.export(export_level=export_level, session=session)
+        return rv
+
+    def _inject_edit_model(self, ep):
+        return self._inject_model(ep, param='changes', model=self._actual.interest_class.tmodel_edit)
+
+    async def edit_item(self, id:int, changes,
+                        user: AuthUserModel = auth_spec(),
+                        export_level: Optional[ExportLevel] = ExportLevel.NORMAL.value):
+        with get_session() as session:
+            item = self._actual.item(id, session=session)
+            result = item.edit(changes, auth_user=user, session=session)
+            rv = item.export(export_level=export_level, auth_user=user, session=session)
         return rv
 
     async def activate_item(self, request: Request, id: int,
+                            background_tasks: BackgroundTasks,
                             user: AuthUserModel = auth_spec()):
         """
         Activate a specified interest.
@@ -186,13 +200,47 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
         """
         with get_session() as session:
             item = self._actual.item(id, session=session)
-            item.activate(auth_user=user, session=session)
+            result, msg = item.activate(background_tasks=background_tasks,
+                                        auth_user=user, session=session)
+        if result:
+            status_code = 200
+        else:
+            status_code = 406
+        return JSONResponse(
+            status_code=status_code,
+            content={'message': msg}
+        )
+
+    async def deactivate_item(self, request: Request, id: int,
+                              user: AuthUserModel = auth_spec()):
+        """
+        Deactivate a specified interest.
+
+        This endpoint enforces interest access control, and executes only if
+        the logged-in user (identified by the access token) has the necessary
+        permissions.
+
+        Activated interests disallow most modifying operations to be carried out
+        on them. Usually, the operations needed to modify the interest in a
+        significant was is only allowed when the interest is new.
+
+        Note that such changes may also (independent of this endpoint) rescind
+        approvals granted to the interest in the past.
+
+        The endpoint parameters are:
+
+         - **id :** The id of the interest to be activated
+         - **user :** The requesting user, identified by the access token.
+        """
+        with get_session() as session:
+            item = self._actual.item(id, session=session)
+            msg = item.deactivate(auth_user=user, session=session)
         return JSONResponse(
             status_code=200,
-            content={'message': f"{self._actual.interest_class.model.role_spec.prefix} "
-                                f"{id} Activated"}
+            content={'message': msg}
         )
 
+
     async def item_members(self, request: Request, id: int,
                            user: AuthUserModel = auth_spec(),
                            include_effective: bool=False,
@@ -273,30 +321,26 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
         with get_session() as session:
             item = self._actual.item(id, session=session)
             item.assign_role(role=role, user=to_user, auth_user=user, session=session)
-            rv = item.export(auth_user=to_user, session=session,
-                             include_roles=True)
+            # TODO This response is probably much heavier than it should be
+            rv = item.export(auth_user=to_user, session=session, export_level=ExportLevel.DETAILED)
         return rv
 
     async def item_parents(self, request: Request, id: int,
                            user: AuthUserModel = auth_spec(),
-                           exclude_limited: bool = True):
+                           export_level: Optional[ExportLevel] = ExportLevel.STUB):
         kwargs = {}
-        if exclude_limited:
-            kwargs['limited'] = False
         with get_session() as session:
             item = self._actual.item(id, session=session)
-            rv = [x.export(auth_user=user, session=session)
+            rv = [x.export(export_level=export_level, auth_user=user, session=session)
                   for x in item.parents(auth_user=user, **kwargs, session=session)]
         return rv
 
     def item_children(self, request: Request, id: int,
                       user: AuthUserModel = auth_spec(),
                       child_type: str = None,
-                      exclude_limited: bool = True):
+                      export_level: Optional[ExportLevel] = ExportLevel.STUB):
         kwargs = {}
         rv = []
-        if exclude_limited:
-            kwargs['limited'] = False
         if child_type:
             kwargs['child_type'] = child_type
         with get_session() as session:
@@ -310,60 +354,91 @@ class InterestLibraryRouterGenerator(ApiRouterGenerator):
                        user: AuthUserModel = auth_spec()):
         with get_session() as session:
             item = self._actual.item(id, session=session)
-            asn = item.add_child(child_id, limited=limited,
-                                 auth_user=user, session=session)
-
-    async def update_item(self):
-        raise NotImplementedError
+            return item.add_child(child_id, limited=limited,
+                                  auth_user=user, session=session)
 
     async def delete_item(self):
         raise NotImplementedError
 
     def generate(self, name):
-        desc = f'{name} Interest API'
+        desc = f'{titleize(singularize(name))} Interest API'
         prefix = self._actual.interest_class.model.role_spec.prefix
+        from tendril import interests
+        parent_models = [interests.type_codes[x].export_tmodel_stub() for x in interests.possible_parents[prefix]]
+
         router = APIRouter(prefix=f'/{name}', tags=[desc],
                            dependencies=[Depends(authn_dependency)])
-        router.add_api_route("", self.items, methods=["GET"],
-                             response_model=List[self._actual.interest_class.tmodel],
-                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])],)
-        router.add_api_route("/new", self.new_items, methods=["GET"],
-                             response_model=List[self._actual.interest_class.tmodel],
-                             dependencies=[auth_spec(scopes=[f'{prefix}:create'])], )
-        router.add_api_route("/{id}", self.item, methods=["GET"],
-                             response_model=self._actual.interest_class.tmodel,
-                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])])
-        router.add_api_route("/create", self._inject_create_model(self.create_item), methods=['PUT'],
-                             response_model=self._actual.interest_class.tmodel,
-                             dependencies=[auth_spec(scopes=[f'{prefix}:create'])], )
-        router.add_api_route("/{id}/activate", self.activate_item, methods=['POST'],
-                             dependencies=[auth_spec(scopes=[f'{prefix}:write'])])
-        router.add_api_route("/{id}/members", self.item_members, methods=["GET"],
-                             response_model=Dict[str, List[MembershipInfoTModel]],
-                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])], )
-        router.add_api_route("/{id}/members/{role}", self.item_role_members, methods=["GET"],
-                             response_model=List[MembershipInfoTModel],
-                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])], )
-        router.add_api_route("/{id}/members/{role}/add", self.item_grant_role, methods=["POST"],
-                             response_model=self._actual.interest_class.tmodel,
-                             dependencies=[auth_spec(scopes=[f'{prefix}:write'])], )
 
-        from tendril import interests
-        parent_models = [interests.type_codes[x].tmodel for x in interests.possible_parents[prefix]]
+        router.add_api_route("", self.items, methods=["GET"],
+                             response_model=List[self._actual.interest_class.export_tmodel_unified()],
+                             response_model_exclude_none=True,
+                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])],)
+
+        if self._actual.enable_creation_api:
+            router.add_api_route("/create", self._inject_create_model(self.create_item), methods=['PUT'],
+                                 response_model=self._actual.interest_class.export_tmodel_unified(),
+                                 response_model_exclude_none=True,
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:create'])], )
+
+        if self._actual.enable_activation_api:
+            router.add_api_route("/new", self.new_items, methods=["GET"],
+                                 response_model=List[self._actual.interest_class.export_tmodel_unified()],
+                                 response_model_exclude_none=True,
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:create'])], )
+
+            if len(parent_models):
+                router.add_api_route("/possible_parents", self.find_possible_parents, methods=['GET'],
+                                     response_model=List[Union[tuple(parent_models)]],
+                                     response_model_exclude_none=True,
+                                     dependencies=[auth_spec(scopes=[f'{prefix}:create'])])
+
+            router.add_api_route("/{id:int}/activate", self.activate_item, methods=['POST'],
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:write'])])
+            router.add_api_route("/{id:int}/deactivate", self.deactivate_item, methods=['POST'],
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:write'])])
+
+        router.add_api_route("/{id:int}", self.item, methods=["GET"],
+                             response_model=self._actual.interest_class.export_tmodel_unified(),
+                             response_model_exclude_none=True,
+                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])])
+
+        router.add_api_route("/{id:int}/edit", self._inject_edit_model(self.edit_item),
+                             methods=["POST"],
+                             response_model=self._actual.interest_class.export_tmodel_unified(),
+                             response_model_exclude_none=True,
+                             dependencies=[auth_spec(scopes=[f'{prefix}:read'])])
+
+        if self._actual.enable_membership_api:
+            router.add_api_route("/{id:int}/members", self.item_members, methods=["GET"],
+                                 response_model=Dict[str, List[MembershipInfoTModel]],
+                                 response_model_exclude_none=True,
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:read'])], )
+            router.add_api_route("/{id:int}/members/{role}", self.item_role_members, methods=["GET"],
+                                 response_model=List[MembershipInfoTModel],
+                                 response_model_exclude_none=True,
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:read'])], )
+
+        if self._actual.enable_membership_edit_api:
+            router.add_api_route("/{id:int}/members/{role}/add", self.item_grant_role, methods=["POST"],
+                                 response_model=self._actual.interest_class.export_tmodel_unified(),
+                                 response_model_exclude_none=True,
+                                 dependencies=[auth_spec(scopes=[f'{prefix}:write'])], )
 
         if len(parent_models):
-            router.add_api_route("/{id}/parents", self.item_parents, methods=["GET"],
+            router.add_api_route("/{id:int}/parents", self.item_parents, methods=["GET"],
                                  response_model=List[Union[tuple(parent_models)]],
+                                 response_model_exclude_none=True,
                                  dependencies=[auth_spec(scopes=[f'{prefix}:read'])],)
 
         ac = self._actual.interest_class.model.role_spec.allowed_children
         if '*' in ac:
             ac = interests.type_codes.keys()
-        child_models = [interests.type_codes[x].tmodel for x in ac]
+        child_models = [interests.type_codes[x].export_tmodel_unified() for x in ac]
         if len(child_models):
-            router.add_api_route("/{id}/children", self.item_children, methods=["GET"],
+            router.add_api_route("/{id:int}/children", self.item_children, methods=["GET"],
                                  response_model=List[Union[tuple(child_models)]],
+                                 response_model_exclude_none=True,
                                  dependencies=[auth_spec(scopes=[f'{prefix}:read'])])
-            router.add_api_route("/{id}/children/add", self.item_add_child, methods=["POST"],
+            router.add_api_route("/{id:int}/children/add", self.item_add_child, methods=["POST"],
                                  dependencies=[auth_spec(scopes=[f'{prefix}:write'])])
         return [router]
