@@ -7,6 +7,7 @@ from sqlalchemy import Boolean
 from sqlalchemy import Integer
 from sqlalchemy import ForeignKey
 from sqlalchemy import UniqueConstraint
+from sqlalchemy.orm import deferred
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import mapped_column
 from sqlalchemy_json import mutable_json_type
@@ -15,7 +16,7 @@ from sqlalchemy.dialects.postgresql import JSONB
 
 from tendril.authz.roles.interests import InterestRoleSpec
 
-from tendril.common.interests.states import InterestLifecycleStatus
+from tendril.common.states import LifecycleStatus
 
 from tendril.utils.db import DeclBase
 from tendril.utils.db import BaseMixin
@@ -41,9 +42,16 @@ class InterestModel(DeclBase, BaseMixin, TimestampMixin):
     name = Column(String(255), nullable=False)
     descriptive_name = Column(String(255), nullable=True)
 
-    status = Column(Enum(InterestLifecycleStatus), nullable=False,
-                    default=InterestLifecycleStatus.NEW)
-    info = Column(mutable_json_type(dbtype=JSONB))
+    status = Column(Enum(LifecycleStatus), nullable=False,
+                    default=LifecycleStatus.NEW)
+    info = deferred(Column(mutable_json_type(dbtype=JSONB)))
+
+    @property
+    def actual(self):
+        if not hasattr(self, '_actual'):
+            from tendril.interests import type_codes
+            self._actual = type_codes[self.type_name](self)
+        return self._actual
 
     # @declared_attr
     # def parent(cls):
@@ -53,12 +61,22 @@ class InterestModel(DeclBase, BaseMixin, TimestampMixin):
     def memberships(cls):
         return relationship("InterestMembershipModel", back_populates='interest', lazy='dynamic')
 
+    @declared_attr
+    def approvals(cls):
+        return relationship("InterestApprovalModel", back_populates='context', lazy='select',
+                            foreign_keys='InterestApprovalModel.interest_id')
+
+    @declared_attr
+    def child_approvals(cls):
+        return relationship("InterestApprovalModel", back_populates='interest', lazy='select',
+                            foreign_keys='InterestApprovalModel.context_id')
+
     @declared_attr
     def children(cls):
         return relationship("InterestModel", secondary="InterestAssociation",
                             primaryjoin="InterestModel.id == InterestAssociationModel.parent_id",
                             secondaryjoin="InterestModel.id == InterestAssociationModel.child_id",
-                            backref="parents")
+                            backref="parents", lazy='dynamic')
 
     # @declared_attr
     # def artefacts(cls):
