@@ -1,18 +1,19 @@
 
 
-from typing import List
+from copy import deepcopy
+from pprint import pformat
 from typing import Optional
 from typing import Literal
 from functools import cached_property
 from pydantic import Field
+from deepdiff import DeepDiff
 
 from tendril.utils.pydantic import TendrilTBaseModel
 
 from tendril.authn.db.controller import get_user_by_id
 from tendril.db.models.interests import InterestModel
-from tendril.db.models.interests import InterestLifecycleStatus
 
-from tendril.common.interests.exceptions import InterestStateException
+from tendril.common.states import LifecycleStatus
 from tendril.common.interests.exceptions import RequiredRoleNotPresent
 from tendril.common.interests.exceptions import RequiredParentNotPresent
 from tendril.common.interests.exceptions import ActivationNotAllowedFromState
@@ -34,6 +35,8 @@ from tendril.authz.roles.interests import require_permission
 from tendril.authz.roles.interests import normalize_role_name
 from tendril.authz.roles.interests import normalize_type_name
 
+from .mixins.export import InterestExportMixin
+
 from tendril.utils.db import with_db
 from tendril.utils import log
 logger = log.get_logger(__name__, log.DEFAULT)
@@ -41,28 +44,28 @@ logger = log.get_logger(__name__, log.DEFAULT)
 
 class InterestBaseCreateTModel(TendrilTBaseModel):
     name: str = Field(..., max_length=255)
-    descriptive_name : Optional[str] = Field(..., max_length=255)
+    descriptive_name: Optional[str] = Field(..., max_length=255)
     type: Literal['interest']
     info: dict
 
 
-class InterestBaseReadTMixin(TendrilTBaseModel):
-    id: int
-    roles: Optional[List[str]]
-    permissions: Optional[List[str]]
-    status: InterestLifecycleStatus
+class InterestBaseEditTModel(TendrilTBaseModel):
+    descriptive_name: Optional[str] = Field(default="", max_length=255)
+    info: Optional[dict]
 
 
-class InterestBaseTModel(InterestBaseCreateTModel,
-                         InterestBaseReadTMixin):
-    ...
-
-
-class InterestBase(object):
+class InterestBase(InterestExportMixin):
     model = InterestModel
+
     tmodel_create = InterestBaseCreateTModel
-    tmodel = InterestBaseTModel
+    tmodel_edit = InterestBaseEditTModel
+
+    # Fields required for creation. Will also be included in NORMAL export.
     additional_fields = []
+    # Fields required for creation only. Will not be included in exports.
+    additional_creation_fields = []
+
+    additional_activation_checks = []
 
     def __init__(self, name, info=None, must_create=False,
                  can_create=True, session=None):
@@ -70,11 +73,15 @@ class InterestBase(object):
         self._descriptive_name = None
         self._info = None
         self._model_instance: InterestModel = None
-        self._status: InterestLifecycleStatus = None
+        self._status: LifecycleStatus = None
         if isinstance(name, InterestModel):
             if must_create:
                 raise AttributeError("Expected a name, not an object")
             self._model_instance = name
+        elif isinstance(name, self.__class__):
+            logger.warning(f"Got a wrapped interest class for {self.__class__}. "
+                           f"Caller should have just used that instead of hitting here!")
+            self._model_instance = name.model_instance
         else:
             self._name = name
             self._info = info or {}
@@ -105,6 +112,15 @@ class InterestBase(object):
         self._descriptive_name = value
         self._commit_to_db(session=session)
 
+    @with_db
+    @require_permission('edit', strip_auth=False, required=False)
+    def edit(self, changes, auth_user=None, session=None):
+        for field_name in changes.__fields__:
+            value = getattr(changes, field_name)
+            if value and hasattr(self.model_instance, field_name):
+                setattr(self.model_instance, field_name, value)
+        session.add(self.model_instance)
+
     @property
     def ident(self):
         return f"{self.__class__.__name__} {self.name}"
@@ -116,6 +132,36 @@ class InterestBase(object):
         else:
             return self._model_instance.info
 
+    @with_db
+    def replace_info(self, info, section=None, context=None,
+                     ignore_changes=None, session=None):
+        session.add(self._model_instance)
+        logger.info(f"Got info for <{self.type_name} {self.id} {self.name}>")
+        old_info = self.info
+        if section:
+            new_info = deepcopy(old_info)
+            new_info[section] = info
+        else:
+            new_info = info
+
+        info_diff = DeepDiff(old_info, new_info,
+                             cache_purge_level=0,
+                             exclude_paths=ignore_changes,
+                             ignore_string_type_changes=True,
+                             ignore_numeric_type_changes=True,
+                             verbose_level=2)
+
+        if info_diff:
+            logger.info(f"Change in info for <{self.type_name} {self.id} {self.name}>. Updating.")
+            logger.debug(f"info diff for <{self.type_name} {self.id} {self.name}> :"
+                         f"\n{pformat(info_diff, depth=2)}")
+            diff_msg = info_diff.to_json()
+            # TODO Publish this to the interest log.
+            self._model_instance.info = new_info
+        else:
+            logger.debug(f"Got identical information for {self.type_name} {self.id} {self.name}, "
+                         f"not updating.")
+
     @property
     def id(self):
         return self._model_instance.id
@@ -131,8 +177,7 @@ class InterestBase(object):
         self._commit_to_db()
 
     @with_db
-    @require_permission('edit')
-    def activate(self, session=None):
+    def _check_activation_requirements(self, session=None):
         if self.status not in self.model.role_spec.activation_requirements['allowed_states']:
             raise ActivationNotAllowedFromState(self.status, self.id, self.name)
         for role in self.model.role_spec.activation_requirements['roles_required']:
@@ -142,9 +187,66 @@ class InterestBase(object):
         if self.model.role_spec.activation_requirements['parent_required']:
             if not len(self.parents(limited=False, session=session)):
                 raise RequiredParentNotPresent(self.id, self.name)
-        logger.info(f"Activating {self.model.type_name} Interest {self.id} {self.name}")
-        self._model_instance.status = InterestLifecycleStatus.ACTIVE
+
+    @property
+    def _additional_activation_checks(self):
+        try:
+            mro = list(self.__class__.__mro__)
+        except AttributeError:
+            print(f"There isn't an __mro__ on {self.__class__}. "
+                  f"This is probably a classic class. We don't support this.")
+            return self.additional_activation_checks
+        rv = []
+        for cls in mro:
+            if hasattr(cls, 'additional_activation_checks'):
+                for check_fn in cls.additional_activation_checks:
+                    rv.append(check_fn)
+        return rv
+
+    @with_db
+    @require_permission('edit', strip_auth=False, required=False)
+    @require_state([LifecycleStatus.NEW, LifecycleStatus.APPROVAL, LifecycleStatus.ACTIVE])
+    def activate(self, background_tasks=None, auth_user=None, session=None):
+        # logger.debug(f"Attempting to activate {self.type_name} {self.id}")
+        if self.model_instance.status == LifecycleStatus.ACTIVE:
+            msg = f"{self.model.type_name} Interest {self.id} {self.name} is already active"
+            logger.info(msg)
+            return False, msg
+
+        # logger.debug(f"Checking Activation Requirements for {self.type_name} {self.id}")
+        self._check_activation_requirements(session=session)
+
+        for check_fn_orig in self._additional_activation_checks:
+            # logger.debug(f"Trying Additional Activation Check for {self.type_name} {self.id} : {check_fn_orig}")
+            if not callable(check_fn_orig):
+                check_fn = getattr(self, check_fn_orig)
+            else:
+                check_fn = check_fn_orig
+            result = check_fn(auth_user=auth_user, session=session)
+            if not result:
+                msg = f"Additional activation check '{check_fn_orig}' failed. " \
+                      f"Will not activate."
+                logger.debug(msg)
+                return False, msg
+            # else:
+            #   logger.debug(f"Additional activation check '{check_fn_orig}' passed. Got {result}.")
+
+        logger.debug(f"Passed all activation checks. Activating {self.type_name} {self.id}")
+        self._model_instance.status = LifecycleStatus.ACTIVE
         session.add(self._model_instance)
+        msg = f"Activated {self.model.type_name} Interest {self.id} {self.name}"
+        logger.info(msg)
+        return True, msg
+
+    @with_db
+    @require_permission('edit', strip_auth=False, required=False)
+    @require_state([LifecycleStatus.ACTIVE, LifecycleStatus.APPROVAL])
+    def deactivate(self, auth_user=None, session=None):
+        self._model_instance.status = LifecycleStatus.NEW
+        session.add(self.model_instance)
+        msg = f"Deactivated {self.model.type_name} Interest {self.id} {self.name}"
+        logger.info(msg)
+        return msg
 
     @property
     def roles(self):
@@ -155,7 +257,7 @@ class InterestBase(object):
         return self._model_instance.role_spec.allowed_children
 
     @with_db
-    @require_state(InterestLifecycleStatus.ACTIVE,
+    @require_state(LifecycleStatus.ACTIVE,
                    exceptions=[(('status', 'NEW'),)])
     @require_permission('add_member', specifier='role', required=False,
                         preprocessor=normalize_role_name, strip_auth=False,
@@ -206,6 +308,11 @@ class InterestBase(object):
 
     @with_db
     def check_user_access(self, user, action, session=None):
+        if hasattr(user, 'id'):
+            user_str = user.id
+        else:
+            user_str = user
+        logger.debug(f"Checking permissions to '{action}' on '{self}' for user '{user_str}'")
         action_roles = self.model.role_spec.get_permitted_roles(action)
         user_roles = set(self.get_user_effective_roles(user, session=session))
         return any(x in user_roles for x in action_roles)
@@ -294,6 +401,16 @@ class InterestBase(object):
             get_parents(self.id, limited=limited, session=session)
         )
 
+    @with_db
+    @require_permission('read', strip_auth=False, required=False)
+    def ancestors(self, auth_user=None, session=None):
+        rv = []
+        parents = self.parents(auth_user=auth_user, session=session)
+        rv.extend(parents)
+        for parent in parents:
+            rv.extend(parent.ancestors(auth_user=auth_user, session=session))
+        return rv
+
     @with_db
     @require_permission('read_children', strip_auth=False, required=False,
                         specifier='child_type', preprocessor=normalize_type_name)
@@ -310,7 +427,7 @@ class InterestBase(object):
         return child.type_name
 
     @with_db
-    @require_state(InterestLifecycleStatus.ACTIVE)
+    @require_state(LifecycleStatus.ACTIVE)
     @require_permission('add_child', specifier=_get_child_type, preprocessor=normalize_type_name)
     def add_child(self, child, limited=False, session=None):
         return add_child(child, self.id, self.type_name,
@@ -341,28 +458,9 @@ class InterestBase(object):
     def artefacts(self, artefact_type, session=None):
         pass
 
-    @with_db
-    @require_permission(action='read', strip_auth=False, required=False)
-    def export(self, session=None, auth_user=None,
-               include_permissions=False,
-               include_roles=False):
-        rv = {
-            'name': self.name,
-            'type': self.type_name,
-            'id': self.id,
-            'info': self.info,
-            'status': self.status,
-            'descriptive_name': self.descriptive_name,
-        }
-        for field in self.additional_fields:
-            rv[field] = getattr(self, field)
-        if include_roles or include_permissions:
-            user_roles = self.get_user_effective_roles(auth_user, session=session)
-        if include_roles:
-            rv['roles'] = sorted(user_roles)
-        if include_permissions:
-            rv['permissions'] = sorted(self.model.role_spec.get_roles_permissions(user_roles))
-        return rv
+    @property
+    def role_spec(self):
+        return self.model.role_spec
 
     @property
     def model_instance(self):
@@ -380,6 +478,8 @@ class InterestBase(object):
         if self._descriptive_name:
             kwargs['descriptive_name'] = self._descriptive_name
         for field in self.additional_fields:
+            if isinstance(field, tuple):
+                field = field[0]
             kwargs[field] = getattr(self, field)
         if self._model_instance:
             kwargs['id'] = self.id
